/*
 * Part of Very Secure FTPd
 * Licence: GPL v2
 * Author: Dmitriy Balashov
 * http_msg.c
 *
 * HTTP messages.
 */

#include "defs.h"
#include "http_msg.h"
#include "tunables.h"
#include "str.h"
#include "filestr.h"
#include "utility.h"
#include "sysutil.h"
//#include "logging.h"

static void http_message_set(struct mystr* p_str, const char* p_filename, const char* p_default);

static struct mystr msg_forbidden_server = INIT_MYSTR;
static struct mystr msg_forbidden_object = INIT_MYSTR;
static struct mystr msg_not_found        = INIT_MYSTR;
static struct mystr msg_browse           = INIT_MYSTR;
struct mystr msg_browse_line             = INIT_MYSTR;
static char* forbidden   = "<h1>Forbidden<h1>";
static char* not_found   = "<h1>Not Found<h1>";
static char* browse      = "<h1>Index of %r</h1><table border=0 cellspacing=2 cellpadding=2>%b</table>";
static char* browse_line = "<tr><td>%f</td><td><a href='%l'>%i</a></td><td>%s</td><td>%T</td></tr>";

void
vsf_http_messages_load()
{
  http_message_set(&msg_forbidden_server, tunable_http_error_403_server_tpl, forbidden);
  http_message_set(&msg_forbidden_object, tunable_http_error_403_tpl, forbidden);
  http_message_set(&msg_not_found, tunable_http_error_404_tpl, not_found);
  http_message_set(&msg_browse, tunable_http_browse_tpl, browse);
  http_message_set(&msg_browse_line, tunable_http_browse_line_tpl, browse_line);
}

const char*
vsf_http_messages_get(const enum EVSFHttpMsgType p_msg_type)
{
  switch (p_msg_type)
  {
    case kVSFHttpMsgType403Serv:
      return str_getbuf(&msg_forbidden_server);
      break;
    case kVSFHttpMsgType403:
      return str_getbuf(&msg_forbidden_object);
      break;
    case kVSFHttpMsgType404:
      return str_getbuf(&msg_not_found);
      break;
    case kVSFHttpMsgType200Browse:
      return str_getbuf(&msg_browse);
      break;
    default:
      bug("unknown message type in vsf_http_messages_get");
      break;
  }
  return 0;
}

static void
http_message_set(struct mystr* p_str, const char* p_filename, const char* p_default)
{
  int retval = 0;
  if (p_filename)
  {
    retval = str_fileread(p_str, p_filename, VSFTP_DATA_BUFSIZE);
  }
  if (!p_filename || vsf_sysutil_retval_is_error(retval))
  {
    str_alloc_text(p_str, p_default);
  }
}

